==> hpalib-0.8.2.tar.gz: hpalib/src/xmul.c <==
/*
   Copyright (C)  2000    Daniel A. Atkinson
   Copyright (C)  2004    Ivano Primi  <ivano.primi@tin.it>    

   This file is part of the HPA Library.

   The HPA Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The HPA Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the HPA Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.
*/

#include "xpre.h"

struct xpr xmul(struct xpr s,struct xpr t)
{ 
  unsigned short pe[XDIM+2],*pa,*pb,*pc,*q0,*q1,h;
  unsigned int m,n,p; short k,e;
  
  q0=(unsigned short *)&s; q1=(unsigned short *)&t;
  e=(*q0&m_exp)-bias; k=(*q1&m_exp)+1;
  if(e>(short)m_exp-k) return x_huge; if((e+=k)<=0) return zero;
  h=(*q0 ^ *q1)&m_sgn;
  for(++q1,k=XDIM,p=n=0L,pc=pe+XDIM+1; k>0 ;--k){
    for(pa=q0+k,pb=q1; pa>q0 ;){
      m=  *pa--; m*= *pb++; n+=(m&0xffffL); p+=(m>>16); }
    *pc-- =n; n=p+(n>>16); p=0L;
   }
  *pc=n;
  if(!(*pc&m_sgn)){ --e; if(e<=0) return zero; lshift(1,pc,XDIM+1);}
  if(e==(short)m_exp) return x_huge;
  *pe=e; *pe|=h; return *(struct xpr *)pe;
}

==> hpalib-0.9.10.tar.gz: hpalib/src/xmul.c <==
/*
   Copyright (C)  2000    Daniel A. Atkinson
   Copyright (C)  2004    Ivano Primi  <ivano.primi@tin.it>    

   This file is part of the HPA Library.

   The HPA Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The HPA Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the HPA Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.
*/

#include "xpre.h"

struct xpr xmul(struct xpr s,struct xpr t)
{ 
  unsigned short pe[XDIM+2],*pa,*pb,*pc,*q0,*q1,h;
  unsigned int m,n,p; short k,e;
  
  q0=(unsigned short *)&s; 
  q1=(unsigned short *)&t;
  e=(*q0&m_exp)-bias; 
  k=(*q1&m_exp)+1;
  if( (xsigerr(e>(short)m_exp-k, XFPOFLOW, NULL)) ) 
    return x_huge; 
  if((e+=k)<=0) 
    return zero;
  h=(*q0 ^ *q1)&m_sgn;
  for(++q1,k=XDIM,p=n=0L,pc=pe+XDIM+1; k>0 ; --k) {
    for(pa=q0+k,pb=q1; pa>q0 ;) {
      m=  *pa--; 
      m*= *pb++; 
      n+=(m&0xffffL); 
      p+=(m>>16); 
    }
    *pc-- =n; 
    n=p+(n>>16); 
    p=0L;
  }
  *pc=n;
  if(!(*pc&m_sgn))
    { 
      --e; 
      if(e<=0) return zero; 
      lshift(1,pc,XDIM+1);
    }
  if( (xsigerr(e==(short)m_exp, XFPOFLOW, NULL)) )
    return x_huge;
  *pe=e; *pe|=h; 
  return *(struct xpr *)pe;
}
